#!/usr/bin/env python
#
# Problem EPage 411
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
import numpy as np; from numpy import asarray, pi, sqrt

from powell import powell

def objective_fn(x):
    """
    The function we want to minimize.
    """
    r, h, b = x
    return pi * r * ( 2 * h + sqrt( b**2 + r**2 ) ) # the surface area 

def constraint_fns(x): 
    """
    A tuple of constraints we want to be <= 0.
    """
    r, h, b = x 
    return ( -r, -h, -b, abs( pi * r**2 * ( b/3. + h ) - 1 ), ) # the last is the volume constraint 

def lagrangian_function(lam): 
    """
    Given a value of lambda returns a function of x that we can minimize with a minimization routine.
    """
    return lambda _ : F(_,lam)

def F(x,lam): 
    """
    Given lambda returns the value of the objective function 
    """
    objective = objective_fn(x)
    penalties = map( lambda _: max(_,0)**2, constraint_fns(x) )
    penalty = sum( penalties ) 
    return objective + lam * penalty 

# Start with an initial value of a Lagrangian value of lambda: 
lam = 1000.0
xStart = asarray( [1/sqrt(pi), 1./2, 3./2] ) # r, h, b 
x, numIter = powell( lagrangian_function(lam), xStart, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)

# Increase lambda: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)
