#!/usr/bin/env python
#
# Problem EPage 411
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
import numpy as np; from numpy import asarray

from powell import powell

def objective_fn(x):
    """
    The function we want to minimize.
    """
    return ( x[0] - 1 )**2 + ( x[1] - 1 )**2 

def constraint_fns(x): 
    """
    A tuple of constraints we want to be <= 0.
    """
    return ( x[0] + x[1] - 1, -x[0] + 0.6 )

def lagrangian_function(lam): 
    """
    Given a value of lambda returns a function of x that we can minimize with a minimization routine.
    """
    return lambda _ : F(_,lam)

def F(x,lam): 
    """
    Given lambda returns the value of the objective function 
    """
    objective = objective_fn(x)
    penalties = map( lambda _: max(_,0)**2, constraint_fns(x) )
    penalty = sum( penalties ) 
    return objective + lam * penalty 

# Start with an initial value of a Lagrangian value of lambda: 
lam = 1000.0
xStart = asarray( [1.0, 1.0] )
x, numIter = powell( lagrangian_function(lam), xStart, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", constraint_fns(x) 
print "Function value= ", F(x,lam)

# Increase lambda: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", constraint_fns(x) 
print "Function value= ", F(x,lam)

# Increase lambda once more: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", constraint_fns(x) 
print "Function value= ", F(x,lam)

# Increase lambda once more: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", constraint_fns(x) 
print "Function value= ", F(x,lam)





