#!/usr/bin/env python
#
# Problem EPage 411
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
import numpy as np; from numpy import asarray

from powell import powell

def objective_fn(x):
    """
    The function we want to minimize.
    """
    return 6 * x[0]**2 + x[1]**3 + x[0] * x[1] 

def constraint_fns(x): 
    """
    A tuple of constraints we want to be <= 0.  
    Now we want y >= -2 or -y <= 2 or -y - 2 <= 0 
    """
    return ( -x[1] - 2 , ) 

def lagrangian_function(lam): 
    """
    Given a value of lambda returns a function of x that we can minimize with a minimization routine.
    """
    return lambda _ : F(_,lam)

def F(x,lam): 
    """
    Given lambda returns the value of the objective function 
    """
    objective = objective_fn(x)
    penalties = map( lambda _: max(_,0)**2, constraint_fns(x) )
    penalty = sum( penalties ) 
    return objective + lam * penalty 

# Start with an initial value of a Lagrangian value of lambda: 
lam = 1.0
xStart = asarray( [0.0, 1.0] )
x, numIter = powell( lagrangian_function(lam), xStart, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", constraint_fns(x) 
print "Function value= ", F(x,lam)

# Increase lambda: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", constraint_fns(x) 
print "Function value= ", F(x,lam)




