#!/usr/bin/env python
#
# Problem EPage 106
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
import sys
import numpy as np; from numpy import asarray, dot, identity, zeros 

from conjGrad import *

def Ax(v):
    n = len(v)
    Ax = zeros(n,dtype=float)
    Ax[0] =                 3*v[0] - 2*v[1] + v[2]
    Ax[1] =       -2*v[0] + 4*v[1] - 2*v[2] + v[3]
    Ax[2] = v[0] - 2*v[1] + 4*v[2] - 2*v[3] + v[4]
    Ax[3] = v[1] - 2*v[2] + 4*v[3] - 2*v[4] + v[5]
    Ax[4] = v[2] - 2*v[3] + 4*v[4] - 2*v[5]
    Ax[5] = v[3] - 2*v[4] + 3*v[5]
    return Ax

def Bx(v):
    n = len(v)
    Bx = zeros(n,dtype=float)
    Bx[0] =                 3*v[0] - 2*v[1] + v[2] + v[5] # Note we add v[5] here ... 
    Bx[1] =       -2*v[0] + 4*v[1] - 2*v[2] + v[3]
    Bx[2] = v[0] - 2*v[1] + 4*v[2] - 2*v[3] + v[4]
    Bx[3] = v[1] - 2*v[2] + 4*v[3] - 2*v[4] + v[5]
    Bx[4] = v[2] - 2*v[3] + 4*v[4] - 2*v[5]
    Bx[5] = v[3] - 2*v[4] + 3*v[5]                 + v[0] # Note we add v[0] here ... 
    return Bx
    
A = asarray( [ [ 3, -2,  1,  0,  0,  0 ], [ -2, 4, -2,  1, 0,  0 ], [ 1, -2, 4, -2,  1, 0 ], 
               [ 0,  1, -2,  4, -2,  1 ], [  0, 0,  1, -2, 4, -2 ], [ 0,  0, 0,  1, -2, 3 ] ], dtype=float )
b = asarray( [ 10, -8, 10, 10, -8, 10 ], dtype=float ) 

# Check that the function Ax is implemented correctly:
# 
e = zeros( (A.shape[0]) )
for ii in range(A.shape[0]):
    e[ii] = 1.
    p_1 = dot(A,e)
    p_2 = Ax(e)
    error = np.linalg.norm( p_1 - p_2 ) 
    if error > 1.e-6:
        print "Error in Ax computation" 
        print ii, error
        print p_1, p_2 
        sys.exit(1)
    e[ii] = 0.

x = zeros( (A.shape[0]), dtype=float ) 
x,numIter = conjGrad(Ax,x,b)
print "Solution to Ax=b; x= ", map( lambda _: round(_,4), x )
print "numIter= ", numIter
assert np.linalg.norm( dot(A,x) - b ) < 1.e-9 


B = asarray( [ [ 3, -2,  1,  0,  0,  1 ], [ -2, 4, -2,  1, 0,  0 ], [ 1, -2, 4, -2,  1, 0 ], 
               [ 0,  1, -2,  4, -2,  1 ], [  0, 0,  1, -2, 4, -2 ], [ 1,  0, 0,  1, -2, 3 ] ], dtype=float )

# Check that the function Bx is implemented correctly:
# 
e = zeros( (B.shape[0]) )
for ii in range(B.shape[0]):
    e[ii] = 1.
    p_1 = dot(B,e)
    p_2 = Bx(e)
    error = np.linalg.norm( p_1 - p_2 ) 
    if error > 1.e-6:
        print "Error in Bx computation" 
        print ii, error
        print p_1, p_2 
        sys.exit(1)
    e[ii] = 0.

x = zeros( (B.shape[0]), dtype=float ) 
x,numIter = conjGrad(Bx,x,b)
print "Solution to Bx=b; x= ", x 
print "numIter= ", numIter
assert np.linalg.norm( dot(B,x) - b ) < 1.e-9 

