#!/usr/bin/env python
#
from numpy import nan, empty 

# Problem EPage 206

thetas = [ 0, 30, 60, 90, 120, 150 ] 
betas = [ 59.96, 56.42, 44.10, 25.72, -0.27, -34.29 ]

def fp_approx(x,delta_t=1):
    """
    Returns an estimate of x'(t) using forward/central/backwards differences of O(h^2) 
    """
    n = len(x)
    x_p = nan*empty(n)

    # Use forward differences for the first location (Table 5.3a):
    # 
    for ii in [0]: 
        x_p[ii] = ( - 3 * x[ii] + 4 * x[ii+1] - x[ii+2] ) / ( 2 * delta_t ) 

    # Use central differences for the middle locations (Table 5.1):
    # 
    for ii in range(1,n-1): 
        x_p[ii] = ( - x[ii-1] + x[ii+1] ) / ( 2 * delta_t ) 

    # Use backwards differences for the last location (Table 5.3b):
    # 
    for ii in [n-1]: 
        x_p[ii] = ( x[ii-2] - 4 * x[ii-1] + 3 * x[ii] ) / ( 2 * delta_t ) 

    return x_p

print fp_approx(betas)



