#!/usr/bin/env python
#
from numpy import sqrt, sin, cos, arange, empty, nan, pi 

# Problem EPage 205

DELTA_THETA = 5
thetas = arange( 0, 180+DELTA_THETA, DELTA_THETA ).astype(float) # in degrees
thetas *= ( pi / 180. ) # in radians 

R = 90
x_of_theta = R * ( cos( thetas ) + sqrt( 2.5**2 - sin( thetas )**2 ) )
x_p_of_theta_exact = R * ( -sin( thetas ) - ( sin( thetas ) * cos( thetas ) ) / sqrt( 2.5**2 - sin( thetas )**2 ) ) 
x_pp_of_theta_exact = R * ( 
    -cos( thetas ) 
    - ( cos( thetas ) * cos( thetas ) ) / sqrt( 2.5**2 - sin( thetas )**2 ) 
    + ( sin( thetas ) * sin( thetas ) ) / sqrt( 2.5**2 - sin( thetas )**2 ) 
    - ( sin( thetas ) * cos( thetas ) )**2 * ( 2.5**2 - sin( thetas )**2 )**(-3./2)  
) 

def fpp_approx_Oh2(x,delta_theta=5.):
    """
    Returns an estimate of x''(theta) using forward/central/backwards differences of O(h^2) 
    """
    n = len(x)
    x_pp = nan*empty(n)

    # Use forward differences O(h^2) for the first location (Table 5.3a):
    # 
    for ii in [0]: 
        x_pp[ii] = ( 2 * x[ii] - 5 * x[ii+1] + 4 * x[ii+2] - x[ii+3] ) / delta_theta**2 

    # Use central differences O(h^2) for the middle locations (Table 5.1):
    # 
    for ii in range(1,n-1): 
        x_pp[ii] = ( x[ii-1] - 2 * x[ii] + x[ii+1] ) / delta_theta**2 

    # Use backwards differences O(h^2) for the last location (Table 5.3a):
    # 
    for ii in [n-1]: 
        x_pp[ii] = ( - x[ii-3] + 4 * x[ii-2] - 5 * x[ii-1] + 2 * x[ii] ) / delta_theta**2 

    return x_pp 


# Compute the approximate derivatives of x(theta):
# 
print fpp_approx_Oh2( x_of_theta, delta_theta=(DELTA_THETA*(pi/180.)) ) 

