#!/usr/bin/env python
#
# Problem Page 318
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
from numpy import zeros,array,pi,sin
import pylab

from bulStoer import *
from newtonRaphson2 import *
from printSoln import * 

def initCond(u):  # Initial values of [y,y',y'''];
                  # use 'u' if unknown
    return array([0.0, u[0], u[1]])

def r(u):  # Boundary condition residuals--see Eq. (8.7)
    r = zeros(len(u))
    X,Y = bulStoer(F,xStart,initCond(u),xStop,H) # Has the solution to our differtial equation at X = [ -1, 0, +1 ] 
    ind_middle = abs( X - 0 ).argmin() # get the indices for x=0
    y = Y[ind_middle,:] # get Y(0)
    r[0] = y[0] # y(0)=0
    ind_right = abs( X - xStop ).argmin() 
    y = Y[ind_right,:] # get Y(1)
    r[1] = y[0] - 1 # y(1)=1
    return r
              
def F(x,y):       # First-order differential equations
    F = zeros(len(y))
    F[0] = y[1]
    F[1] = y[2] 
    F[2] = -2*y[2] - sin(y[0])
    return F

xStart = -1.0       # Start of integration
xStop  = +1.0       # End of integration 
H      = 0.01       # When integrating from [-1,+1] make sure produce a valid estimate of the solution at x=0.0 
freq = 0            # Printout frequency

u = array([-1.0, -1.0])      # Initial guess for {u}
u = newtonRaphson2(r,u,1.0e-4)
X,Y = bulStoer(F,xStart,initCond(u),xStop,H)
printSoln(X,Y,freq)

if True: 
    pylab.plot( X, Y[:,0], color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
