#!/usr/bin/python
#
from numpy import zeros,array,sin,cos,pi
import pylab 
from bulStoer import *
from newtonRaphson2 import *
from printSoln import *

m = 20 
c = 3.2*1.e-4
g = 9.80665

def initCond(u):  # Initial values of [x,y,x',y']
                  # use 'u' if unknown u[0] = theta, u[1] = v_0
    return array([0.0, 0.0, u[1]*cos(u[0]), u[1]*sin(u[0])])

def r(u):  # Boundary condition residuals--see Eq. (8.7)
    r = zeros(len(u))
    X,Y = bulStoer(F,xStart,initCond(u),xStop,H)
    y = Y[len(Y) - 1]
    r[0] = y[0] - 8000 # x(10) = 8000 meters 
    r[1] = y[1]        # y(10) = 0 
    return r
              
def F(x,y):  # First-order differential equations                   
    x_t, y_t, x_dot_t, y_dot_t = y 
    F = zeros(len(y))
    v = sqrt( x_dot_t**2 + y_dot_t**2 ) 
    F[0] = x_dot_t # =dx/dt 
    F[1] = y_dot_t # =dy/dt
    F[2] = -(c/m)*v*x_dot_t # =d^2x/dt^2
    F[3] = -(c/m)*v*y_dot_t - g # =d^2y/dt^2 
    return F

xStart = 0.0               # Start of integration        
xStop = 10.0               # End of integration
u = array([pi/4, 800])     # Initial guess for {u}
H = 0.05                   # Printout incremant
freq = 0                   # Printout frequency
u = newtonRaphson2(r,u,1.0e-4)
print "u[0] (deg)= %10.6f; u[1]= %10.6f" % ((u[0]*180)/pi,u[1])
X,Y = bulStoer(F,xStart,initCond(u),xStop,H)
printSoln(X,Y,freq)

if True: 
    pylab.plot( Y[:,0], Y[:,1], color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
