#!/usr/bin/python
#
from numpy import zeros,array
import pylab 
from bulStoer import *
from newtonRaphson2 import *
from printSoln import *

def initCond(u):  # Initial values of [y,y',y'',y'''];
                  # use 'u' if unknown
    return array([0.0, u[0], 0.0, u[1]])

def r(u):  # Boundary condition residuals--see Eq. (8.7)
    r = zeros(len(u))
    X,Y = bulStoer(F,xStart,initCond(u),xStop,H)
    y = Y[len(Y) - 1]
    r[0] = y[0] # y(1) = 0 
    r[1] = y[2] # y''(1) = 0 
    return r
              
def F(x,y):  # First-order differential equations                   
    F = zeros(len(y))
    F[0] = y[1]
    F[1] = y[2]
    F[2] = y[3]
    F[3] = beta * y[2] + 1.
    return F

beta = 1.65929 
beta *= -1 
xStart = 0.0               # Start of integration        
xStop = 1.0                # End of integration
u = array([-1.0, +1.0])    # Initial guess for {u} = y'(0), y'''(0) 
H = 0.05                   # Printout incremant
freq = 0                   # Printout frequency
u = newtonRaphson2(r,u,1.0e-4)
X,Y = bulStoer(F,xStart,initCond(u),xStop,H)
print "H= %5.3f; beta= %10.6f, max displacement= %10.6f" % (H,beta,max(Y[:,0]))
printSoln(X,Y,freq)

if True: 
    pylab.plot( X, Y[:,0], color='black', label='approx. sol.' )
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
