#!/usr/bin/python
#
from numpy import zeros,ones,array,arange,exp
from LUdecomp3 import *

#
# y''(x) = - 2 y' - y 
# y(0) = 0
# y(1) = 1
#
# Exact y = x e^{1-x} 
#

def equations(x,h,m): # Set up finite difference eqs.
    """
    (1-h) y_{i-1} + (-2 + h^2 ) y_i + (1+h) y_{i+1} = 0 
    """
    h2 = h*h

    # Form the parts of A = [c\d\e]:
    c = (1-h) * ones(m)
    d = (-2 + h2) * ones(m+1)
    e = (1+h) * ones(m)
    d[0] = 1.0 # first equation is: y_0 = 0
    e[0] = 0.0 
    d[m] = 1.0 # last equation is: y_m = 1
    c[m-1] = 0.0 

    # Form the parts of b: 
    b = zeros(m+1) 
    b[0] = 0.0
    b[m] = 1.0
    return c,d,e,b

xStart = 0.0        # x at left end
xStop = 1.0         # x at right end
m = 10              # Number of mesh spaces
h = (xStop - xStart)/m
x = arange(xStart,xStop + h,h)
c,d,e,b = equations(x,h,m)
c,d,e = LUdecomp3(c,d,e)
y = LUsolve3(c,d,e,b)
print "\n        x              y"
for i in [0,m]: 
    print "%14.5e %14.5e" %(x[i],y[i])

import pylab 

if True: 
    pylab.plot( x, y, color='black', label='approx. sol.' )
    pylab.hold('on')
    pylab.plot( x, x * exp(1-x), color='green', label='true sol.')
    pylab.xlabel('x'); pylab.ylabel('y(x)')
    pylab.grid('on')
    pylab.axis('tight')
    pylab.legend(loc='best')
    pylab.show() 
